	page    132,60
	opt	mex

;============================================================================
;	STM_MAIN.ASM 
;
;	Framework for STM driver software for EVM56002.
;	Copyright (C) Jrgen Mller, juergen@e-basteln.de
;
; 	jm 3.10.99
;	jm 6.11.99	interrupt priorities defined 
;	jm 8.11.99	overflow checks for fast memory segments
;	jm 6.1.2000	stm_timer renamed to stm_portb; old test code removed
;	jm 16.1.2000	separated scan generation and Z feedback loop,
;			to reduce feedback pass-through delay
;
;============================================================================

	; Memory model defines

P_INT	equ	$40			; Internal P memory start
P_EXT   equ     $200			; External P memory start
X_INT	equ	$0			; Internal X memory start
X_EXT	equ	$100			; External X memory -- identical to P space!
Y_INT	equ	$0			; Internal Y memory start
Y_EXT	equ	$100			; External Y memory start

START   equ     P_EXT			; Main program memory
STACK	equ	Y_INT			; Stack in Y memory grows up

	; Initialize memory pointers
	; P (default) to be used for IRQ vectors

	org	ph:START		; P memory for normal program
	org	pl:P_INT		; P memory for IRQ handlers
PSTART	equ	*			; label used for range check (end of stm_main)
	org	y:Y_EXT			; Y memory used for standard variables
	org	x:X_INT			; X memory used for "fast" variables
XSTART	equ	*			; label used for range check (end of stm_main)

	
	; Reset and IRQ vectors

	org     p:0                     ; RESET vector
	jmp     main

	DUP	$30			; empty IRQ vectors
	nop
	ENDM

	; Interrupt priority levels

ipl_hst	equ	0			; host port, not used
ipl_ssi	equ	3			; ssi, highest priority
ipl_sci	equ	2			; sci
ipl_tim	equ	1			; timer

ipr_val	equ	 (ipl_hst<<10)+(ipl_ssi<<12)+(ipl_sci<<14)+(ipl_tim<<16)

;----------------------------------------------------------------------------


;----------------------------------------------------------------------------
;	DSP56002 on-chip peripheral addresses
;	(from ada_init.asm)
;----------------------------------------------------------------------------

IPR             equ     $FFFF   	; Interrupt Priority Register
BCR             equ     $FFFE   	; Bus Control Register
PLL             equ     $FFFD   	; PLL Control Register
SSIDR           equ     $FFEF   	; SSI Data Register
SSISR           equ     $FFEE   	; SSI Status Register
CRB             equ     $FFED   	; SSI Control Register B
CRA             equ     $FFEC   	; SSI Control Register A
PCD             equ     $FFE5   	; Port C Data Register
PBD             equ     $FFE4   	; Port B Data Register
PCDDR           equ     $FFE3   	; Port C Data Direction Register
PBDDR           equ     $FFE2   	; Port B Data Direction Register
PCC             equ     $FFE1   	; Port C Control Register
PBC             equ     $FFE0   	; Port B Control Register

TCSR		equ	$FFDE		; Timer Control and Status
TCR		equ	$FFDF		; Timer Count

SCR		equ	$FFF0		; SCI control register
SCCR		equ	$FFF2		; SCI clock control register
SRX		equ	$FFF4		; SCI receive register
STX		equ	$FFF4		; SCI transmit register

;---------------------------------------------------------------------------
;      Include specific drivers
;---------------------------------------------------------------------------

	include	"stm_portb.asm"		; timer and port B: LEDs and stepper motor
	include	"stm_sci.asm"		; buffered serial I/O
	include	"stm_ssi.asm"		; buffered serial I/O

	include "stm_cmd.asm"		; command interpreter

;---------------------------------------------------------------------------
;      General EVM 56002 initialization
;---------------------------------------------------------------------------
	org     ph:
main
	movep   #$26100E,x:PLL          ; set PLL for MPY of 15x (60 MHz)
	movep   #$0000,x:BCR            ; zero wait states in all ext. memory
	ori     #3,mr                   ; disable interrupts
	movec   #0,sp                   ; clear hardware stack pointer
	move    #0,omr                  ; mode 0: enable int. P:RAM, rst=0000
	move    #STACK,r6               ; initialize stack pointer
	move    #-1,m6                  ; linear addressing
	movep   #ipr_val,x:IPR    	; set timer interrupt priority levels

	; initialize hardware drivers
	jsr	ssi_init		; initialize SSI first (overwrites PCC reg!)
					; ssi_init enables interrupts!
	jsr	portb_init
	jsr	sci_init

	; other initializations
	jsr	cmd_init

	; ready to go
	; interrupts have been enabled by ssi_init!
	; andi    #$FC,mr         	; enable interrupts

	; main loop

	jsr	ssi_newscan		; initialize scan
_loop	
	;ssi_fsync			; wait for frame sync to complete

	ssi_slot 5			; wait for slot 5 rx to complete
	set_test	1,1		; set test pin
	jsr	ssi_feedback		; Z feedback loop
	set_test	1,0		; set test pin
	jsr	ssi_scan		; X/Y scan 

	move	x:rx_crcount,a		; wait until CR (complete command) received
	tst	a
	jeq	_loop			; nothing yet
	jsr	cmd_getline		; command line complete: process
	jmp	_loop

;---------------------------------------------------------------------------
;      Check for fast memory segment overflows
;---------------------------------------------------------------------------
	
	; check x < X_EXT
	org x:
	IF	(*-XSTART)>X_EXT
	FAIL	'jm: Fast X data memory overflow'
	ENDIF

	; check pl < P_EXT
	org pl:
	IF	(*-PSTART)>P_EXT
	FAIL	'jm: Fast program memory overflow'
	ENDIF
	end
